/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 transmission library
* Version		: v1.1
* Last updated	: 30.07.07
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

***************************************************************************/

#include "lib_dmx_out.h"
#include <util/delay.h>

// ********************* local definitions *********************
		uint16_t 	gCurDmxCh;		//current DMX channel
		uint8_t		gDmxState;

enum {BREAK, STARTB, DATA};

// *************** DMX Transmission Initialisation ****************
void init_DMX(void)
{
//Ports
DDRD |= (1<<2)|(1<<1);
PORTD |= (1<<2)|(1<<1);				//enable transmission

//USART
UBRRH  = 0;
UBRRL  = ((F_CPU/4000000UL)-1);			//250kbaud, 8N2
UCSRC |= (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB |= (1<<TXEN)|(1<<TXCIE);
UDR    = 0;							//start USART

//Data
gDmxState= BREAK;					//start with break
}



// ****************** DMX Transmission ISR ********************

ISR (USART_TX_vect)
{
uint8_t DmxState= gDmxState;

if (DmxState == BREAK)
	{
	UBRRH  = 0;
	UBRRL  = (F_CPU/1600000UL);					//90.9kbaud
	UDR    = 0;								//send break
	gDmxState= STARTB;
	}
else if (DmxState == STARTB)
	{
	UBRRH  = 0;
	UBRRL  = ((F_CPU/4000000UL)-1);				//250kbaud
	UDR    = 0;								//send start byte
	gDmxState= DATA;
	gCurDmxCh= 0;
	}
else
	{
	_delay_us(IBG);
	uint16_t CurDmxCh= gCurDmxCh;
	UDR= DmxField[CurDmxCh++];				//send data
	if (CurDmxCh == sizeof(DmxField)) gDmxState= BREAK; //new break if all ch sent
	else gCurDmxCh= CurDmxCh;
	}
}

